# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The trigger commands module.

These commands are used in the following models: SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


# pylint: disable=too-few-public-methods
class TriggerTimerItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``trigger.timer[N]`` command tree.

    Constants:
        - ``.EVENT_ID``: The trigger timer event number (1 to 8).
    """

    EVENT_ID = "trigger.timer[N].EVENT_ID"
    """str: The trigger timer event number (1 to 8)."""

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )


# pylint: disable=too-few-public-methods
class TriggerGeneratorItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``trigger.generator[N]`` command tree.

    Constants:
        - ``.EVENT_ID``: The trigger event generated by  trigger event generator 1.
    """

    EVENT_ID = "trigger.generator[N].EVENT_ID"
    """str: The trigger event generated by  trigger event generator 1."""

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )


# pylint: disable=too-few-public-methods
class TriggerBlenderItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``trigger.blender[N]`` command tree.

    Constants:
        - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
          output line after a collection of events is detected.
    """

    EVENT_ID = "trigger.blender[N].EVENT_ID"
    """str: Selects the event that causes a trigger to be asserted on the digital output line after a collection of events is detected."""  # noqa: E501

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )


class Trigger(BaseTSPCmd):
    r"""The ``trigger`` command tree.

    Constants:
        - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
          output line as a \*TRG command received on the remote interface.

    Properties/methods:
        - ``.blender``: The ``trigger.blender[N]`` command tree.
        - ``.generator``: The ``trigger.generator[N]`` command tree.
        - ``.timer``: The ``trigger.timer[N]`` command tree.
    """

    EVENT_ID = "trigger.EVENT_ID"
    r"""str: Selects the event that causes a trigger to be asserted on the digital output line as a \*TRG command received on the remote interface."""  # noqa: E501

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "trigger") -> None:
        super().__init__(device, cmd_syntax)
        self._blender: Dict[int, TriggerBlenderItem] = DefaultDictPassKeyToFactory(
            lambda x: TriggerBlenderItem(device, f"{self._cmd_syntax}.blender[{x}]")
        )
        self._generator: Dict[int, TriggerGeneratorItem] = DefaultDictPassKeyToFactory(
            lambda x: TriggerGeneratorItem(device, f"{self._cmd_syntax}.generator[{x}]")
        )
        self._timer: Dict[int, TriggerTimerItem] = DefaultDictPassKeyToFactory(
            lambda x: TriggerTimerItem(device, f"{self._cmd_syntax}.timer[{x}]")
        )

    @property
    def blender(self) -> Dict[int, TriggerBlenderItem]:
        """Return the ``trigger.blender[N]`` command tree.

        Constants:
            - ``.EVENT_ID``: Selects the event that causes a trigger to be asserted on the digital
              output line after a collection of events is detected.
        """
        return self._blender

    @property
    def generator(self) -> Dict[int, TriggerGeneratorItem]:
        """Return the ``trigger.generator[N]`` command tree.

        Constants:
            - ``.EVENT_ID``: The trigger event generated by  trigger event generator 1.
        """
        return self._generator

    @property
    def timer(self) -> Dict[int, TriggerTimerItem]:
        """Return the ``trigger.timer[N]`` command tree.

        Constants:
            - ``.EVENT_ID``: The trigger timer event number (1 to 8).
        """
        return self._timer



"""The trigger commands module.

These commands are used in the following models:
AWG5200

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - TRIGger:IMMediate {ATRigger|BTRigger}
    - TRIGger:IMPedance <impedance>[,<input_trigger>]? [<input_trigger>]
    - TRIGger:INTerval <NR3>
    - TRIGger:INTerval?
    - TRIGger:LEVel <NRf>[,ATRigger|BTRigger]? [ATRigger|BTRigger]
    - TRIGger:MODE {SYNChronous}[,<input_trigger>]? <input_trigger>
    - TRIGger:SLOPe {POSitive|NEGative}[,<input_trigger>]? [<input_trigger>]
    - TRIGger:SOURce {EXTernal|INTernal}
    - TRIGger:SOURce?
    - TRIGger:WVALue {FIRSt}
    - TRIGger:WVALue?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class TriggerWvalue(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:WVALue`` command.

    **Description:**
        - This command sets or returns the channel's output state when in the Waiting-for-trigger
          mode. This value is applied to all channels and markers.

    **Usage:**
        - Using the ``.query()`` method will send the ``TRIGger:WVALue?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:WVALue?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:WVALue value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:WVALue {FIRSt}
        - TRIGger:WVALue?

    **Info:**
        - ``FIRSt`` specifies the first value of the waveform as the output level.
        - ``*RST`` sets this to ZERO.
    """


class TriggerSource(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SOURce`` command.

    **Description:**
        - This command sets or returns the trigger source.

    **Usage:**
        - Using the ``.query()`` method will send the ``TRIGger:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SOURce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SOURce value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:SOURce {EXTernal|INTernal}
        - TRIGger:SOURce?

    **Info:**
        - ``EXTernal`` selects external trigger as the trigger source.INTernal select internal
          interval timing as the trigger source.
        - ``*RST`` sets this to EXT.
    """


class TriggerSlope(SCPICmdWrite):
    r"""The ``TRIGger:SLOPe`` command.

    **Description:**
        - This command sets or returns the polarity of the external trigger slope. Use this command
          to set the polarity in modes other than gated mode.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``TRIGger:SLOPe value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:SLOPe {POSitive|NEGative}[,<input_trigger>]? [<input_trigger>]

    **Info:**
        - ``POSitive`` specifies a trigger on the rising edge of the external trigger signal.
          NEGative specifies a trigger on the falling edge of the external trigger
          signal.``<input_trigger>`` ::= {ATRigger\|BTRigger}, defaults to ATR if not specified.
        - ``*RST`` sets all external trigger slopes to POSitive.
    """


class TriggerMode(SCPICmdWrite):
    """The ``TRIGger:MODE`` command.

    **Description:**
        - This command sets or returns the trigger timing used for the specified A or B external
          trigger source. SYNChronous is the only available trigger mode.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``TRIGger:MODE value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:MODE {SYNChronous}[,<input_trigger>]? <input_trigger>

    **Info:**
        - ``SYNChronous`` : Synchronous triggering. This trigger type is used to synchronize with
          external devices.
        - ``*RST`` sets this to SYNchronous.
    """


class TriggerLevel(SCPICmdWrite):
    """The ``TRIGger:LEVel`` command.

    **Description:**
        - This command sets or returns the external trigger input level (threshold).

    **Usage:**
        - Using the ``.write(value)`` method will send the ``TRIGger:LEVel value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:LEVel <NRf>[,ATRigger|BTRigger]? [ATRigger|BTRigger]

    **Info:**
        - ``ATRigger`` selects trigger input A. BTRigger selects trigger input B. Defaults to ATR if
          not specified.
        - ``*RST`` sets this to 1.4 V.
    """


class TriggerInterval(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:INTerval`` command.

    **Description:**
        - This command sets or returns the internal trigger interval.

    **Usage:**
        - Using the ``.query()`` method will send the ``TRIGger:INTerval?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:INTerval?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:INTerval value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:INTerval <NR3>
        - TRIGger:INTerval?
    """


class TriggerImpedance(SCPICmdWrite):
    """The ``TRIGger:IMPedance`` command.

    **Description:**
        - This command sets or returns the external trigger impedance. It applies only to the
          external trigger.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``TRIGger:IMPedance value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:IMPedance <impedance>[,<input_trigger>]? [<input_trigger>]

    **Info:**
        - ``*RST`` sets this to 50.
    """


class TriggerImmediate(SCPICmdWrite):
    """The ``TRIGger:IMMediate`` command.

    **Description:**
        - This command generates a trigger A or B event. If a trigger is not specified, the command
          is then equivalent to the TRG command.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``TRIGger:IMMediate value`` command.

    **SCPI Syntax:**

    ::

        - TRIGger:IMMediate {ATRigger|BTRigger}
    """


#  pylint: disable=too-many-instance-attributes
class Trigger(SCPICmdRead):
    """The ``TRIGger`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``TRIGger?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.impedance``: The ``TRIGger:IMPedance`` command.
        - ``.interval``: The ``TRIGger:INTerval`` command.
        - ``.level``: The ``TRIGger:LEVel`` command.
        - ``.mode``: The ``TRIGger:MODE`` command.
        - ``.slope``: The ``TRIGger:SLOPe`` command.
        - ``.source``: The ``TRIGger:SOURce`` command.
        - ``.wvalue``: The ``TRIGger:WVALue`` command.
        - ``.immediate``: The ``TRIGger:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "TRIGger") -> None:
        super().__init__(device, cmd_syntax)
        self._impedance = TriggerImpedance(device, f"{self._cmd_syntax}:IMPedance")
        self._interval = TriggerInterval(device, f"{self._cmd_syntax}:INTerval")
        self._level = TriggerLevel(device, f"{self._cmd_syntax}:LEVel")
        self._mode = TriggerMode(device, f"{self._cmd_syntax}:MODE")
        self._slope = TriggerSlope(device, f"{self._cmd_syntax}:SLOPe")
        self._source = TriggerSource(device, f"{self._cmd_syntax}:SOURce")
        self._wvalue = TriggerWvalue(device, f"{self._cmd_syntax}:WVALue")
        self._immediate = TriggerImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def impedance(self) -> TriggerImpedance:
        """Return the ``TRIGger:IMPedance`` command.

        **Description:**
            - This command sets or returns the external trigger impedance. It applies only to the
              external trigger.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``TRIGger:IMPedance value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:IMPedance <impedance>[,<input_trigger>]? [<input_trigger>]

        **Info:**
            - ``*RST`` sets this to 50.
        """
        return self._impedance

    @property
    def interval(self) -> TriggerInterval:
        """Return the ``TRIGger:INTerval`` command.

        **Description:**
            - This command sets or returns the internal trigger interval.

        **Usage:**
            - Using the ``.query()`` method will send the ``TRIGger:INTerval?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:INTerval?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:INTerval value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:INTerval <NR3>
            - TRIGger:INTerval?
        """
        return self._interval

    @property
    def level(self) -> TriggerLevel:
        """Return the ``TRIGger:LEVel`` command.

        **Description:**
            - This command sets or returns the external trigger input level (threshold).

        **Usage:**
            - Using the ``.write(value)`` method will send the ``TRIGger:LEVel value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:LEVel <NRf>[,ATRigger|BTRigger]? [ATRigger|BTRigger]

        **Info:**
            - ``ATRigger`` selects trigger input A. BTRigger selects trigger input B. Defaults to
              ATR if not specified.
            - ``*RST`` sets this to 1.4 V.
        """
        return self._level

    @property
    def mode(self) -> TriggerMode:
        """Return the ``TRIGger:MODE`` command.

        **Description:**
            - This command sets or returns the trigger timing used for the specified A or B external
              trigger source. SYNChronous is the only available trigger mode.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``TRIGger:MODE value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:MODE {SYNChronous}[,<input_trigger>]? <input_trigger>

        **Info:**
            - ``SYNChronous`` : Synchronous triggering. This trigger type is used to synchronize
              with external devices.
            - ``*RST`` sets this to SYNchronous.
        """
        return self._mode

    @property
    def slope(self) -> TriggerSlope:
        r"""Return the ``TRIGger:SLOPe`` command.

        **Description:**
            - This command sets or returns the polarity of the external trigger slope. Use this
              command to set the polarity in modes other than gated mode.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``TRIGger:SLOPe value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:SLOPe {POSitive|NEGative}[,<input_trigger>]? [<input_trigger>]

        **Info:**
            - ``POSitive`` specifies a trigger on the rising edge of the external trigger signal.
              NEGative specifies a trigger on the falling edge of the external trigger
              signal.``<input_trigger>`` ::= {ATRigger\|BTRigger}, defaults to ATR if not specified.
            - ``*RST`` sets all external trigger slopes to POSitive.
        """
        return self._slope

    @property
    def source(self) -> TriggerSource:
        """Return the ``TRIGger:SOURce`` command.

        **Description:**
            - This command sets or returns the trigger source.

        **Usage:**
            - Using the ``.query()`` method will send the ``TRIGger:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SOURce?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SOURce value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:SOURce {EXTernal|INTernal}
            - TRIGger:SOURce?

        **Info:**
            - ``EXTernal`` selects external trigger as the trigger source.INTernal select internal
              interval timing as the trigger source.
            - ``*RST`` sets this to EXT.
        """
        return self._source

    @property
    def wvalue(self) -> TriggerWvalue:
        """Return the ``TRIGger:WVALue`` command.

        **Description:**
            - This command sets or returns the channel's output state when in the
              Waiting-for-trigger mode. This value is applied to all channels and markers.

        **Usage:**
            - Using the ``.query()`` method will send the ``TRIGger:WVALue?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:WVALue?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:WVALue value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:WVALue {FIRSt}
            - TRIGger:WVALue?

        **Info:**
            - ``FIRSt`` specifies the first value of the waveform as the output level.
            - ``*RST`` sets this to ZERO.
        """
        return self._wvalue

    @property
    def immediate(self) -> TriggerImmediate:
        """Return the ``TRIGger:IMMediate`` command.

        **Description:**
            - This command generates a trigger A or B event. If a trigger is not specified, the
              command is then equivalent to the TRG command.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``TRIGger:IMMediate value`` command.

        **SCPI Syntax:**

        ::

            - TRIGger:IMMediate {ATRigger|BTRigger}
        """
        return self._immediate



"""Base Power Supply Unit (PSU) device driver module.

PSUs include PI devices.
"""
from abc import ABC
from typing import Tuple, Union

from tm_devices.drivers.pi.pi_device import PIDevice
from tm_devices.drivers.pi.signal_sources.signal_source import SignalSource
from tm_devices.helpers import DeviceTypes


class PowerSupplyUnit(PIDevice, ABC):
    """Base Power Supply Unit (PSU) device driver."""

    _DEVICE_TYPE = DeviceTypes.PSU.value

    ################################################################################################
    # Public Methods
    ################################################################################################
    def expect_esr(self, esr: Union[int, str], error_string: str = "") -> Tuple[bool, str]:
        r"""Check for the expected number of errors and output string.

        Sends the ``*ESR?`` and SYSTEM:ERROR? queries.

        Args:
            esr: Expected ``*ESR?`` value
            error_string: Expected error buffer string.
                Multiple errors should be separated by a \n character

        Returns:
            Boolean indicating if the check passed or failed and a string with the results.
        """
        return SignalSource.expect_esr(self, esr, error_string)  # type: ignore

    def get_eventlog_status(self) -> Tuple[bool, str]:
        """Help function for getting the eventlog status.

        Returns:
            Boolean indicating no error, String containing concatenated contents of event log.
        """
        return SignalSource.get_eventlog_status(self)  # type: ignore



"""The select commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SELect:B<x> {0|1|OFF|ON}
    - SELect:B<x>?
    - SELect:CH<x> {<NR1>|OFF|ON}
    - SELect:CH<x>?
    - SELect:CONTROl {CH<x>|MATH<x>|REF<x>}
    - SELect:CONTROl?
    - SELect:D<x> {<NR1>|OFF|ON}
    - SELect:D<x>?
    - SELect:DALL <NR1>
    - SELect:DIGTraces:COMbination <nr1>
    - SELect:DIGTraces:LISt <Dx>
    - SELect:DIGTraces:LISt?
    - SELect:REF<x> {ON|OFF|<NR1>}
    - SELect:REF<x>?
    - SELect?
"""
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedChannel,
    ValidatedDigitalBit,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SelectRefItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:REF<x>`` command.

    **Description:**
        - This command turns on the display of a specified waveform and also resets the acquisition.
          The query returns whether the channel is on or off but does not indicate whether it is the
          selected waveform. WFM can be a channel, math, or reference waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:REF<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:REF<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:REF<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:REF<x> {ON|OFF|<NR1>}
        - SELect:REF<x>?

    **Info:**
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectDigtracesList(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:DIGTraces:LISt`` command.

    **Description:**
        - This command turns on the specified digital channels or returns the list of digital
          channels that are on.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:DIGTraces:LISt?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:DIGTraces:LISt?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:DIGTraces:LISt value`` command.

    **SCPI Syntax:**

    ::

        - SELect:DIGTraces:LISt <Dx>
        - SELect:DIGTraces:LISt?
    """


class SelectDigtracesCombination(SCPICmdWrite):
    """The ``SELect:DIGTraces:COMbination`` command.

    **Description:**
        - This command turns on the digital channels that have binary digits as 1. The binary digits
          are obtained from the set decimal value.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SELect:DIGTraces:COMbination value``
          command.

    **SCPI Syntax:**

    ::

        - SELect:DIGTraces:COMbination <nr1>

    **Info:**
        - ``nr1`` is a decimal integer.
    """


class SelectDigtraces(SCPICmdRead):
    """The ``SELect:DIGTraces`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:DIGTraces?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:DIGTraces?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.combination``: The ``SELect:DIGTraces:COMbination`` command.
        - ``.list``: The ``SELect:DIGTraces:LISt`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._combination = SelectDigtracesCombination(device, f"{self._cmd_syntax}:COMbination")
        self._list = SelectDigtracesList(device, f"{self._cmd_syntax}:LISt")

    @property
    def combination(self) -> SelectDigtracesCombination:
        """Return the ``SELect:DIGTraces:COMbination`` command.

        **Description:**
            - This command turns on the digital channels that have binary digits as 1. The binary
              digits are obtained from the set decimal value.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SELect:DIGTraces:COMbination value`` command.

        **SCPI Syntax:**

        ::

            - SELect:DIGTraces:COMbination <nr1>

        **Info:**
            - ``nr1`` is a decimal integer.
        """
        return self._combination

    @property
    def list(self) -> SelectDigtracesList:
        """Return the ``SELect:DIGTraces:LISt`` command.

        **Description:**
            - This command turns on the specified digital channels or returns the list of digital
              channels that are on.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:DIGTraces:LISt?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:DIGTraces:LISt?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:DIGTraces:LISt value``
              command.

        **SCPI Syntax:**

        ::

            - SELect:DIGTraces:LISt <Dx>
            - SELect:DIGTraces:LISt?
        """
        return self._list


class SelectDall(SCPICmdWrite):
    """The ``SELect:DALL`` command.

    **Description:**
        - This command sets the displayed state of all the digital inputs.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SELect:DALL value`` command.

    **SCPI Syntax:**

    ::

        - SELect:DALL <NR1>

    **Info:**
        - ``<NR1> = 0`` turns off the display of all the digital inputs; any other value turns on
          the display of all the digital inputs.
        - ``OFF`` turns off the display of all the digital inputs.
        - ``ON`` displays all the digital inputs.
    """


class SelectDigitalBit(ValidatedDigitalBit, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:D<x>`` command.

    **Description:**
        - Turns on the display of the digital channel <x> and resets the acquisition. <x > is the
          channel number, which can be 0-15. The query returns whether the channel is on or off but
          does not indicate whether it is the selected waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:D<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:D<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:D<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:D<x> {<NR1>|OFF|ON}
        - SELect:D<x>?

    **Info:**
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns on
          the display of the specified waveform.
    """


class SelectControl(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CONTROl`` command.

    **Description:**
        - <wfm> This command sets or queries the waveform that is selected as the implied recipient
          of channel-related commands that support legacy-style programs. The command form also
          performs the equivalent of a ``SELECT:WFM ON`` command. This command is equivalent to
          selecting Measurement Setup from the Measure menu and either viewing or setting the Source
          waveform.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:CONTROl?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CONTROl?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CONTROl value`` command.

    **SCPI Syntax:**

    ::

        - SELect:CONTROl {CH<x>|MATH<x>|REF<x>}
        - SELect:CONTROl?

    **Info:**
        - ``CH<x>`` selects the specified channel waveform as the waveform that is affected by the
          front panel controls. The x variable can be expressed as an integer ranging from 1 through
          4.
        - ``MATH<x>`` selects the specified math waveform as the waveform that is affected by the
          front panel controls. The x variable can be expressed as an integer ranging from 1 through
          4.
        - ``REF<x>`` selects the specified reference waveform as the waveform that is affected by
          the front panel controls. The x variable can be expressed as an integer ranging from 1
          through 4.
    """


class SelectChannel(ValidatedChannel, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CH<x>`` command.

    **Description:**
        - This command sets or queries the displayed state of the specified channel waveform. The x
          can be channel 1 through 4.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:CH<x> {<NR1>|OFF|ON}
        - SELect:CH<x>?

    **Info:**
        - ``<NR1> = 0`` turns off the display of the specified channel waveform; any other value
          turns on the display of the specified waveform.
        - ``OFF`` turns off the display of the indicated channel waveform.
        - ``ON`` displays the indicated channel waveform.
    """


class SelectBItem(ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:B<x>`` command.

    **Description:**
        - This command sets or queries the display state for the bus specified by x. The value of x
          can range from 1 through 16.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect:B<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:B<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:B<x> value`` command.

    **SCPI Syntax:**

    ::

        - SELect:B<x> {0|1|OFF|ON}
        - SELect:B<x>?

    **Info:**
        - ``0`` turns off the display of the specified bus; any other value turns on the display of
          the specified bus.
        - ``1`` turns on the display of the specified bus.
        - ``OFF`` turns off the display of the indicated bus.
        - ``ON`` turns on the display of the indicated bus.
    """


class Select(SCPICmdRead):
    """The ``SELect`` command.

    **Description:**
        - Queries which waveforms are displayed.

    **Usage:**
        - Using the ``.query()`` method will send the ``SELect?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SELect?

    Properties:
        - ``.b``: The ``SELect:B<x>`` command.
        - ``.ch``: The ``SELect:CH<x>`` command.
        - ``.control``: The ``SELect:CONTROl`` command.
        - ``.d``: The ``SELect:D<x>`` command.
        - ``.dall``: The ``SELect:DALL`` command.
        - ``.digtraces``: The ``SELect:DIGTraces`` command tree.
        - ``.ref``: The ``SELect:REF<x>`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SELect") -> None:
        super().__init__(device, cmd_syntax)
        self._b: Dict[int, SelectBItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectBItem(device, f"{self._cmd_syntax}:B{x}")
        )
        self._control = SelectControl(device, f"{self._cmd_syntax}:CONTROl")
        self._dall = SelectDall(device, f"{self._cmd_syntax}:DALL")
        self._digtraces = SelectDigtraces(device, f"{self._cmd_syntax}:DIGTraces")
        self._ch: Dict[int, SelectChannel] = DefaultDictPassKeyToFactory(
            lambda x: SelectChannel(device, f"{self._cmd_syntax}:CH{x}")
        )
        self._ref: Dict[int, SelectRefItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectRefItem(device, f"{self._cmd_syntax}:REF{x}")
        )
        self._d: Dict[int, SelectDigitalBit] = DefaultDictPassKeyToFactory(
            lambda x: SelectDigitalBit(device, f"{self._cmd_syntax}:D{x}")
        )

    @property
    def b(self) -> Dict[int, SelectBItem]:
        """Return the ``SELect:B<x>`` command.

        **Description:**
            - This command sets or queries the display state for the bus specified by x. The value
              of x can range from 1 through 16.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:B<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:B<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:B<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:B<x> {0|1|OFF|ON}
            - SELect:B<x>?

        **Info:**
            - ``0`` turns off the display of the specified bus; any other value turns on the display
              of the specified bus.
            - ``1`` turns on the display of the specified bus.
            - ``OFF`` turns off the display of the indicated bus.
            - ``ON`` turns on the display of the indicated bus.
        """
        return self._b

    @property
    def control(self) -> SelectControl:
        """Return the ``SELect:CONTROl`` command.

        **Description:**
            - <wfm> This command sets or queries the waveform that is selected as the implied
              recipient of channel-related commands that support legacy-style programs. The command
              form also performs the equivalent of a ``SELECT:WFM ON`` command. This command is
              equivalent to selecting Measurement Setup from the Measure menu and either viewing or
              setting the Source waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:CONTROl?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CONTROl?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CONTROl value`` command.

        **SCPI Syntax:**

        ::

            - SELect:CONTROl {CH<x>|MATH<x>|REF<x>}
            - SELect:CONTROl?

        **Info:**
            - ``CH<x>`` selects the specified channel waveform as the waveform that is affected by
              the front panel controls. The x variable can be expressed as an integer ranging from 1
              through 4.
            - ``MATH<x>`` selects the specified math waveform as the waveform that is affected by
              the front panel controls. The x variable can be expressed as an integer ranging from 1
              through 4.
            - ``REF<x>`` selects the specified reference waveform as the waveform that is affected
              by the front panel controls. The x variable can be expressed as an integer ranging
              from 1 through 4.
        """
        return self._control

    @property
    def dall(self) -> SelectDall:
        """Return the ``SELect:DALL`` command.

        **Description:**
            - This command sets the displayed state of all the digital inputs.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SELect:DALL value`` command.

        **SCPI Syntax:**

        ::

            - SELect:DALL <NR1>

        **Info:**
            - ``<NR1> = 0`` turns off the display of all the digital inputs; any other value turns
              on the display of all the digital inputs.
            - ``OFF`` turns off the display of all the digital inputs.
            - ``ON`` displays all the digital inputs.
        """
        return self._dall

    @property
    def digtraces(self) -> SelectDigtraces:
        """Return the ``SELect:DIGTraces`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:DIGTraces?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:DIGTraces?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.combination``: The ``SELect:DIGTraces:COMbination`` command.
            - ``.list``: The ``SELect:DIGTraces:LISt`` command.
        """
        return self._digtraces

    @property
    def ch(self) -> Dict[int, SelectChannel]:
        """Return the ``SELect:CH<x>`` command.

        **Description:**
            - This command sets or queries the displayed state of the specified channel waveform.
              The x can be channel 1 through 4.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:CH<x> {<NR1>|OFF|ON}
            - SELect:CH<x>?

        **Info:**
            - ``<NR1> = 0`` turns off the display of the specified channel waveform; any other value
              turns on the display of the specified waveform.
            - ``OFF`` turns off the display of the indicated channel waveform.
            - ``ON`` displays the indicated channel waveform.
        """
        return self._ch

    @property
    def ref(self) -> Dict[int, SelectRefItem]:
        """Return the ``SELect:REF<x>`` command.

        **Description:**
            - This command turns on the display of a specified waveform and also resets the
              acquisition. The query returns whether the channel is on or off but does not indicate
              whether it is the selected waveform. WFM can be a channel, math, or reference
              waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:REF<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:REF<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:REF<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:REF<x> {ON|OFF|<NR1>}
            - SELect:REF<x>?

        **Info:**
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._ref

    @property
    def d(self) -> Dict[int, SelectDigitalBit]:
        """Return the ``SELect:D<x>`` command.

        **Description:**
            - Turns on the display of the digital channel <x> and resets the acquisition. <x > is
              the channel number, which can be 0-15. The query returns whether the channel is on or
              off but does not indicate whether it is the selected waveform.

        **Usage:**
            - Using the ``.query()`` method will send the ``SELect:D<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:D<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:D<x> value`` command.

        **SCPI Syntax:**

        ::

            - SELect:D<x> {<NR1>|OFF|ON}
            - SELect:D<x>?

        **Info:**
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``<NR1>`` = 0 turns off the display of the specified waveform; any other value turns
              on the display of the specified waveform.
        """
        return self._d



"""The lic commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - LIC:UNINSTALL? <QString>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdReadWithArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class LicUninstall(SCPICmdReadWithArguments):
    """The ``LIC:UNINSTALL`` command.

    **Description:**
        - Returns the exit license indicated for the user to return to their TekAMS account. Active
          licenses can be specified by their nomenclature. TransactionIDs can be used to specify an
          active license or a previously uninstalled license. In either case, the exit-license is
          returned as block-data.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``LIC:UNINSTALL? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``LIC:UNINSTALL? argument``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - LIC:UNINSTALL? <QString>

    **Info:**
        - ``<QString>`` is the nomenclature of an active license or a TransactionIDs to specify an
          active license or a previously uninstalled license.
    """


class Lic(SCPICmdRead):
    """The ``LIC`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``LIC?`` query.
        - Using the ``.verify(value)`` method will send the ``LIC?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.uninstall``: The ``LIC:UNINSTALL`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "LIC") -> None:
        super().__init__(device, cmd_syntax)
        self._uninstall = LicUninstall(device, f"{self._cmd_syntax}:UNINSTALL")

    @property
    def uninstall(self) -> LicUninstall:
        """Return the ``LIC:UNINSTALL`` command.

        **Description:**
            - Returns the exit license indicated for the user to return to their TekAMS account.
              Active licenses can be specified by their nomenclature. TransactionIDs can be used to
              specify an active license or a previously uninstalled license. In either case, the
              exit-license is returned as block-data.

        **Usage:**
            - Using the ``.query(argument)`` method will send the ``LIC:UNINSTALL? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``LIC:UNINSTALL? argument`` query and raise an AssertionError if the returned value
              does not match ``value``.

        **SCPI Syntax:**

        ::

            - LIC:UNINSTALL? <QString>

        **Info:**
            - ``<QString>`` is the nomenclature of an active license or a TransactionIDs to specify
              an active license or a previously uninstalled license.
        """
        return self._uninstall



# pylint: disable=too-many-lines
"""The filesystem commands module.

These commands are used in the following models:
LPD6, MSO2, MSO4, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - FILESystem:COPy {<source_file_path>,<destination_file_path>}
    - FILESystem:CWD {<new_working_directory_path>}
    - FILESystem:CWD?
    - FILESystem:DELEte <file_path>
    - FILESystem:DIR?
    - FILESystem:HOMEDir?
    - FILESystem:LDIR?
    - FILESystem:MKDir <directory_path>
    - FILESystem:MOUNT:DRIVE <QString>? <QString>
    - FILESystem:MOUNT:TEKDrive <QString>? <QString>
    - FILESystem:READFile <QString>
    - FILESystem:REName <old_file_path>,<new_file_path>
    - FILESystem:RMDir <directory_path>
    - FILESystem:TEKDrive:CODE:EXPirytime?
    - FILESystem:TEKDrive:CODE:STATus?
    - FILESystem:TEKDrive:CODE?
    - FILESystem:UNMOUNT:DRIve <QString>
    - FILESystem:UNMOUNT:TEKDrive <QString>
    - FILESystem:WRITEFile <file_path>,<data>
    - FILESystem?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class FilesystemWritefile(SCPICmdWrite):
    """The ``FILESystem:WRITEFile`` command.

    **Description:**
        - This command (no query form) writes the specified block data to the specified file on the
          instruments file system. If the destination file cannot be written, an error event is
          posted.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:WRITEFile value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:WRITEFile <file_path>,<data>

    **Info:**
        - ``<file path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
        - ``<data>`` is the specified block data to be written.
    """


class FilesystemUnmountTekdrive(SCPICmdWrite):
    """The ``FILESystem:UNMOUNT:TEKDrive`` command.

    **Description:**
        - This command unmounts the TekDrive specified by the quoted string argument and the drive
          name is case insensitive.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:UNMOUNT:TEKDrive value``
          command.

    **SCPI Syntax:**

    ::

        - FILESystem:UNMOUNT:TEKDrive <QString>

    **Info:**
        - ``<QString>`` specifies the TekDrive to unmount.
    """

    _WRAP_ARG_WITH_QUOTES = True


class FilesystemUnmountDrive(SCPICmdWrite):
    """The ``FILESystem:UNMOUNT:DRIve`` command.

    **Description:**
        - This command unmounts the USB drive specified by the quoted string argument.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:UNMOUNT:DRIve value``
          command.

    **SCPI Syntax:**

    ::

        - FILESystem:UNMOUNT:DRIve <QString>

    **Info:**
        - ``<QString>`` is a quoted string that specifies which USB drive to unmount. String is a
          case insensitive single letter followed by a colon.
    """

    _WRAP_ARG_WITH_QUOTES = True


class FilesystemUnmount(SCPICmdRead):
    """The ``FILESystem:UNMOUNT`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:UNMOUNT?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:UNMOUNT?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.drive``: The ``FILESystem:UNMOUNT:DRIve`` command.
        - ``.tekdrive``: The ``FILESystem:UNMOUNT:TEKDrive`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._drive = FilesystemUnmountDrive(device, f"{self._cmd_syntax}:DRIve")
        self._tekdrive = FilesystemUnmountTekdrive(device, f"{self._cmd_syntax}:TEKDrive")

    @property
    def drive(self) -> FilesystemUnmountDrive:
        """Return the ``FILESystem:UNMOUNT:DRIve`` command.

        **Description:**
            - This command unmounts the USB drive specified by the quoted string argument.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:UNMOUNT:DRIve value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:UNMOUNT:DRIve <QString>

        **Info:**
            - ``<QString>`` is a quoted string that specifies which USB drive to unmount. String is
              a case insensitive single letter followed by a colon.
        """
        return self._drive

    @property
    def tekdrive(self) -> FilesystemUnmountTekdrive:
        """Return the ``FILESystem:UNMOUNT:TEKDrive`` command.

        **Description:**
            - This command unmounts the TekDrive specified by the quoted string argument and the
              drive name is case insensitive.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:UNMOUNT:TEKDrive value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:UNMOUNT:TEKDrive <QString>

        **Info:**
            - ``<QString>`` specifies the TekDrive to unmount.
        """
        return self._tekdrive


class FilesystemTekdriveCodeStatus(SCPICmdRead):
    """The ``FILESystem:TEKDrive:CODE:STATus`` command.

    **Description:**
        - This command returns status of short code.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:TEKDrive:CODE:STATus?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive:CODE:STATus?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:TEKDrive:CODE:STATus?
    """


class FilesystemTekdriveCodeExpirytime(SCPICmdRead):
    """The ``FILESystem:TEKDrive:CODE:EXPirytime`` command.

    **Description:**
        - This command returns expiry time of short code. It is the absolute time that the expiry
          command returns. For example, if ``2:11`` pm is the time the user initiated the TekDrive
          mounting, then the expiry query returns + 5 minutes (``2:16`` pm).

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:TEKDrive:CODE:EXPirytime?``
          query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive:CODE:EXPirytime?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:TEKDrive:CODE:EXPirytime?
    """


class FilesystemTekdriveCode(SCPICmdRead):
    """The ``FILESystem:TEKDrive:CODE`` command.

    **Description:**
        - This command returns short code in string format. This code must be entered (or pasted) at
          http://drive.tekcloud.com/activate. After the code is entered click the Activate button to
          complete the mounting of the TekDrive.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:TEKDrive:CODE?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive:CODE?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:TEKDrive:CODE?

    Properties:
        - ``.expirytime``: The ``FILESystem:TEKDrive:CODE:EXPirytime`` command.
        - ``.status``: The ``FILESystem:TEKDrive:CODE:STATus`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._expirytime = FilesystemTekdriveCodeExpirytime(
            device, f"{self._cmd_syntax}:EXPirytime"
        )
        self._status = FilesystemTekdriveCodeStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def expirytime(self) -> FilesystemTekdriveCodeExpirytime:
        """Return the ``FILESystem:TEKDrive:CODE:EXPirytime`` command.

        **Description:**
            - This command returns expiry time of short code. It is the absolute time that the
              expiry command returns. For example, if ``2:11`` pm is the time the user initiated the
              TekDrive mounting, then the expiry query returns + 5 minutes (``2:16`` pm).

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:TEKDrive:CODE:EXPirytime?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``FILESystem:TEKDrive:CODE:EXPirytime?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:TEKDrive:CODE:EXPirytime?
        """
        return self._expirytime

    @property
    def status(self) -> FilesystemTekdriveCodeStatus:
        """Return the ``FILESystem:TEKDrive:CODE:STATus`` command.

        **Description:**
            - This command returns status of short code.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:TEKDrive:CODE:STATus?``
              query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive:CODE:STATus?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:TEKDrive:CODE:STATus?
        """
        return self._status


class FilesystemTekdrive(SCPICmdRead):
    """The ``FILESystem:TEKDrive`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:TEKDrive?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.code``: The ``FILESystem:TEKDrive:CODE`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._code = FilesystemTekdriveCode(device, f"{self._cmd_syntax}:CODE")

    @property
    def code(self) -> FilesystemTekdriveCode:
        """Return the ``FILESystem:TEKDrive:CODE`` command.

        **Description:**
            - This command returns short code in string format. This code must be entered (or
              pasted) at http://drive.tekcloud.com/activate. After the code is entered click the
              Activate button to complete the mounting of the TekDrive.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:TEKDrive:CODE?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive:CODE?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:TEKDrive:CODE?

        Sub-properties:
            - ``.expirytime``: The ``FILESystem:TEKDrive:CODE:EXPirytime`` command.
            - ``.status``: The ``FILESystem:TEKDrive:CODE:STATus`` command.
        """
        return self._code


class FilesystemRmdir(SCPICmdWrite):
    """The ``FILESystem:RMDir`` command.

    **Description:**
        - This command (no query form) deletes a named directory. The directory must be empty.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:RMDir value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:RMDir <directory_path>

    **Info:**
        - ``<directory_path>`` is a quoted string that defines the folder name and path. If the
          folder path is within the current working directory, you need only specify the folder
          name.
    """


class FilesystemRename(SCPICmdWrite):
    """The ``FILESystem:REName`` command.

    **Description:**
        - This command (no query form) assigns a new name to an existing file or folder.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:REName value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:REName <old_file_path>,<new_file_path>

    **Info:**
        - ``<old_file_path>`` is a quoted string that defines the file or folder name and path. If
          the path is within the current working directory, you need only specify the file or folder
          name.
        - ``<new_file_path>`` is a quoted string that defines the file or folder name and path. If
          the path is within the current working directory, you need only specify the file or folder
          name.
    """


class FilesystemReadfile(SCPICmdWrite):
    """The ``FILESystem:READFile`` command.

    **Description:**
        - This command writes the contents of the specified file to the current interface. If the
          specified file does not exist or is not readable, an appropriate error event is posted.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:READFile value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:READFile <QString>

    **Info:**
        - ``<QString>`` is a quoted string that defines the file name and path. If the file path is
          within the current working directory, you need only specify the file name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class FilesystemMountTekdrive(SCPICmdWrite):
    """The ``FILESystem:MOUNT:TEKDrive`` command.

    **Description:**
        - This command mounts the TekDrive specified by the quoted string arguments. The argument
          must contain the drive name, AutoDisconnectMode, RestrictToCurrentIP, and
          AutoDisconnectTime. The drive name is the TekDrive name to be mounted. It is case
          insensitive. There are three options available for AutoDisconnectMode: Select Power Cycle
          to unmount the TekDrive after power cycling the oscilloscope. There is no time restriction
          when this option is selected Select Never to mount the TekDrive connection permanently.
          Select Custom to disconnect the TekDrive after a chosen duration. The default selection is
          Power Cycle. RestrictToCurrentIP restricts the connection to current network IP only. This
          may be used for additional network security. AutoDisconnectTime is the required time for
          the Auto Disconnect. The TekDrive gets disconnected automatically from the instrument
          after the specified time. The duration is in hours. The minimum is 0.25 hours and the
          maximum is 744 hours. The query form of this command returns whether or not the specified
          TekDrive is mounted.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:MOUNT:TEKDrive value``
          command.

    **SCPI Syntax:**

    ::

        - FILESystem:MOUNT:TEKDrive <QString>? <QString>

    **Info:**
        - ``<QString>`` provides the information needed to mount the TekDrive and must have the
          drive name, AutoDisconnectMode, RestrictToCurrentIP, and AutoDisconnectTime.
    """


class FilesystemMountDrive(SCPICmdWrite):
    """The ``FILESystem:MOUNT:DRIVE`` command.

    **Description:**
        - The command form mounts a network drive specified by the quoted string argument. The
          quoted string argument is a semicolon separated list of the following fields: Drive name -
          The drive name to be mounted. It is a case insensitive single letter followed by a colon.
          The drive name must be a letter between 'L:' and 'Z:', inclusive. Server identity - The
          server identity is the DNS name of the server or the IP address of the server. Path - The
          path to be mounted (e.g. /level1/level2/mydirectory). User name - The user name for the
          drive. User password - The password for the drive. Domain name - The domain/workgroup of
          the target mount. Verbose - The verbose option to capture mount failure messages. Domain
          name, user name, user password, and verbose are optional and are only used for mounts
          requiring SMB/CIFS interworking (MS Windows and MacOS). The query form returns a 0 or 1 to
          indicate that the drive name (quoted string) is currently mounted or not. A return of 1
          indicates the drive is mounted. A return of 0 indicated the drive is not mounted.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:MOUNT:DRIVE value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:MOUNT:DRIVE <QString>? <QString>
    """


class FilesystemMount(SCPICmdRead):
    """The ``FILESystem:MOUNT`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:MOUNT?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.drive``: The ``FILESystem:MOUNT:DRIVE`` command.
        - ``.tekdrive``: The ``FILESystem:MOUNT:TEKDrive`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._drive = FilesystemMountDrive(device, f"{self._cmd_syntax}:DRIVE")
        self._tekdrive = FilesystemMountTekdrive(device, f"{self._cmd_syntax}:TEKDrive")

    @property
    def drive(self) -> FilesystemMountDrive:
        """Return the ``FILESystem:MOUNT:DRIVE`` command.

        **Description:**
            - The command form mounts a network drive specified by the quoted string argument. The
              quoted string argument is a semicolon separated list of the following fields: Drive
              name - The drive name to be mounted. It is a case insensitive single letter followed
              by a colon. The drive name must be a letter between 'L:' and 'Z:', inclusive. Server
              identity - The server identity is the DNS name of the server or the IP address of the
              server. Path - The path to be mounted (e.g. /level1/level2/mydirectory). User name -
              The user name for the drive. User password - The password for the drive. Domain name -
              The domain/workgroup of the target mount. Verbose - The verbose option to capture
              mount failure messages. Domain name, user name, user password, and verbose are
              optional and are only used for mounts requiring SMB/CIFS interworking (MS Windows and
              MacOS). The query form returns a 0 or 1 to indicate that the drive name (quoted
              string) is currently mounted or not. A return of 1 indicates the drive is mounted. A
              return of 0 indicated the drive is not mounted.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:MOUNT:DRIVE value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:MOUNT:DRIVE <QString>? <QString>
        """
        return self._drive

    @property
    def tekdrive(self) -> FilesystemMountTekdrive:
        """Return the ``FILESystem:MOUNT:TEKDrive`` command.

        **Description:**
            - This command mounts the TekDrive specified by the quoted string arguments. The
              argument must contain the drive name, AutoDisconnectMode, RestrictToCurrentIP, and
              AutoDisconnectTime. The drive name is the TekDrive name to be mounted. It is case
              insensitive. There are three options available for AutoDisconnectMode: Select Power
              Cycle to unmount the TekDrive after power cycling the oscilloscope. There is no time
              restriction when this option is selected Select Never to mount the TekDrive connection
              permanently. Select Custom to disconnect the TekDrive after a chosen duration. The
              default selection is Power Cycle. RestrictToCurrentIP restricts the connection to
              current network IP only. This may be used for additional network security.
              AutoDisconnectTime is the required time for the Auto Disconnect. The TekDrive gets
              disconnected automatically from the instrument after the specified time. The duration
              is in hours. The minimum is 0.25 hours and the maximum is 744 hours. The query form of
              this command returns whether or not the specified TekDrive is mounted.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:MOUNT:TEKDrive value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:MOUNT:TEKDrive <QString>? <QString>

        **Info:**
            - ``<QString>`` provides the information needed to mount the TekDrive and must have the
              drive name, AutoDisconnectMode, RestrictToCurrentIP, and AutoDisconnectTime.
        """
        return self._tekdrive


class FilesystemMkdir(SCPICmdWrite):
    """The ``FILESystem:MKDir`` command.

    **Description:**
        - This command (no query form) creates a new directory.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:MKDir value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:MKDir <directory_path>

    **Info:**
        - ``<directory_path>`` is a quoted string that specifies the directory to create.
    """


class FilesystemLdir(SCPICmdRead):
    """The ``FILESystem:LDIR`` command.

    **Description:**
        - Returns a comma separated list of every file, file size, type, modification date and time,
          and directory in the folder referred to by the ``FILESYSTEM:CWD`` command. This is
          different than the ``:DIR`` query in that it provides a long output format with the file
          size, type, and modification date/time. Each entry is a semicolon separated list: < file
          name>;<type>;<size in bytes>;<date>;<time>

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:LDIR?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:LDIR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:LDIR?
    """


class FilesystemHomedir(SCPICmdRead):
    """The ``FILESystem:HOMEDir`` command.

    **Description:**
        - This query returns the current user's home directory.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:HOMEDir?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:HOMEDir?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:HOMEDir?
    """


class FilesystemDir(SCPICmdRead):
    """The ``FILESystem:DIR`` command.

    **Description:**
        - This query-only command returns a comma separated list of quoted strings. Each string
          contains the name of a file or directory in the folder referred to by the
          ``FILESYSTEM:CWD`` command.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:DIR?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:DIR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem:DIR?
    """


class FilesystemDelete(SCPICmdWrite):
    """The ``FILESystem:DELEte`` command.

    **Description:**
        - This command (no query form) deletes a named file or directory from a mass storage device.
          Once removed, the data in that file or directory can no longer be accessed. If the
          specified file is a directory, it must be empty before it can be deleted.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:DELEte value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:DELEte <file_path>

    **Info:**
        - ``<file_path>`` is a quoted string that defines the file name and path. If the file path
          is within the current working directory, you need only specify the file name.
    """


class FilesystemCwd(SCPICmdWrite, SCPICmdRead):
    """The ``FILESystem:CWD`` command.

    **Description:**
        - This command sets or queries the current working directory. CWD is short for Current
          Working Directory. It changes the directory (folder) that the other FILESystem commands
          operate on.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem:CWD?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem:CWD?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``FILESystem:CWD value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:CWD {<new_working_directory_path>}
        - FILESystem:CWD?

    **Info:**
        - ``<new_working_directory_path>`` is a quoted string that defines the current working; a
          directory name can be up to 128 characters.
    """


class FilesystemCopy(SCPICmdWrite):
    """The ``FILESystem:COPy`` command.

    **Description:**
        - This command (no query form) copies a named file to a new file. The new file might be in a
          totally separate directory than the old file. You can only copy one file at a time using
          this command. Wild card characters are not allowed.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FILESystem:COPy value`` command.

    **SCPI Syntax:**

    ::

        - FILESystem:COPy {<source_file_path>,<destination_file_path>}

    **Info:**
        - ``<source_file_path>`` is a quoted string that defines the file name and path or
          directory. If the file path is within the current working directory, you need only specify
          the file name.
        - ``<destination_file_path>`` is a quoted string that defines the file name and path. If the
          file path is within the current working directory, you need only specify the file name.
    """


#  pylint: disable=too-many-instance-attributes
class Filesystem(SCPICmdRead):
    """The ``FILESystem`` command.

    **Description:**
        - This query-only command returns the directory listing of the current working directory.
          This query is the same as the ``FILESystem:DIR?`` query.

    **Usage:**
        - Using the ``.query()`` method will send the ``FILESystem?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESystem?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - FILESystem?

    Properties:
        - ``.copy``: The ``FILESystem:COPy`` command.
        - ``.cwd``: The ``FILESystem:CWD`` command.
        - ``.delete``: The ``FILESystem:DELEte`` command.
        - ``.dir``: The ``FILESystem:DIR`` command.
        - ``.homedir``: The ``FILESystem:HOMEDir`` command.
        - ``.ldir``: The ``FILESystem:LDIR`` command.
        - ``.mkdir``: The ``FILESystem:MKDir`` command.
        - ``.mount``: The ``FILESystem:MOUNT`` command tree.
        - ``.readfile``: The ``FILESystem:READFile`` command.
        - ``.rename``: The ``FILESystem:REName`` command.
        - ``.rmdir``: The ``FILESystem:RMDir`` command.
        - ``.tekdrive``: The ``FILESystem:TEKDrive`` command tree.
        - ``.unmount``: The ``FILESystem:UNMOUNT`` command tree.
        - ``.writefile``: The ``FILESystem:WRITEFile`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "FILESystem") -> None:
        super().__init__(device, cmd_syntax)
        self._copy = FilesystemCopy(device, f"{self._cmd_syntax}:COPy")
        self._cwd = FilesystemCwd(device, f"{self._cmd_syntax}:CWD")
        self._delete = FilesystemDelete(device, f"{self._cmd_syntax}:DELEte")
        self._dir = FilesystemDir(device, f"{self._cmd_syntax}:DIR")
        self._homedir = FilesystemHomedir(device, f"{self._cmd_syntax}:HOMEDir")
        self._ldir = FilesystemLdir(device, f"{self._cmd_syntax}:LDIR")
        self._mkdir = FilesystemMkdir(device, f"{self._cmd_syntax}:MKDir")
        self._mount = FilesystemMount(device, f"{self._cmd_syntax}:MOUNT")
        self._readfile = FilesystemReadfile(device, f"{self._cmd_syntax}:READFile")
        self._rename = FilesystemRename(device, f"{self._cmd_syntax}:REName")
        self._rmdir = FilesystemRmdir(device, f"{self._cmd_syntax}:RMDir")
        self._tekdrive = FilesystemTekdrive(device, f"{self._cmd_syntax}:TEKDrive")
        self._unmount = FilesystemUnmount(device, f"{self._cmd_syntax}:UNMOUNT")
        self._writefile = FilesystemWritefile(device, f"{self._cmd_syntax}:WRITEFile")

    @property
    def copy(self) -> FilesystemCopy:
        """Return the ``FILESystem:COPy`` command.

        **Description:**
            - This command (no query form) copies a named file to a new file. The new file might be
              in a totally separate directory than the old file. You can only copy one file at a
              time using this command. Wild card characters are not allowed.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:COPy value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:COPy {<source_file_path>,<destination_file_path>}

        **Info:**
            - ``<source_file_path>`` is a quoted string that defines the file name and path or
              directory. If the file path is within the current working directory, you need only
              specify the file name.
            - ``<destination_file_path>`` is a quoted string that defines the file name and path. If
              the file path is within the current working directory, you need only specify the file
              name.
        """
        return self._copy

    @property
    def cwd(self) -> FilesystemCwd:
        """Return the ``FILESystem:CWD`` command.

        **Description:**
            - This command sets or queries the current working directory. CWD is short for Current
              Working Directory. It changes the directory (folder) that the other FILESystem
              commands operate on.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:CWD?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:CWD?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``FILESystem:CWD value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:CWD {<new_working_directory_path>}
            - FILESystem:CWD?

        **Info:**
            - ``<new_working_directory_path>`` is a quoted string that defines the current working;
              a directory name can be up to 128 characters.
        """
        return self._cwd

    @property
    def delete(self) -> FilesystemDelete:
        """Return the ``FILESystem:DELEte`` command.

        **Description:**
            - This command (no query form) deletes a named file or directory from a mass storage
              device. Once removed, the data in that file or directory can no longer be accessed. If
              the specified file is a directory, it must be empty before it can be deleted.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:DELEte value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:DELEte <file_path>

        **Info:**
            - ``<file_path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
        """
        return self._delete

    @property
    def dir(self) -> FilesystemDir:
        """Return the ``FILESystem:DIR`` command.

        **Description:**
            - This query-only command returns a comma separated list of quoted strings. Each string
              contains the name of a file or directory in the folder referred to by the
              ``FILESYSTEM:CWD`` command.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:DIR?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:DIR?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:DIR?
        """
        return self._dir

    @property
    def homedir(self) -> FilesystemHomedir:
        """Return the ``FILESystem:HOMEDir`` command.

        **Description:**
            - This query returns the current user's home directory.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:HOMEDir?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:HOMEDir?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:HOMEDir?
        """
        return self._homedir

    @property
    def ldir(self) -> FilesystemLdir:
        """Return the ``FILESystem:LDIR`` command.

        **Description:**
            - Returns a comma separated list of every file, file size, type, modification date and
              time, and directory in the folder referred to by the ``FILESYSTEM:CWD`` command. This
              is different than the ``:DIR`` query in that it provides a long output format with the
              file size, type, and modification date/time. Each entry is a semicolon separated list:
              < file name>;<type>;<size in bytes>;<date>;<time>

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:LDIR?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:LDIR?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - FILESystem:LDIR?
        """
        return self._ldir

    @property
    def mkdir(self) -> FilesystemMkdir:
        """Return the ``FILESystem:MKDir`` command.

        **Description:**
            - This command (no query form) creates a new directory.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:MKDir value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:MKDir <directory_path>

        **Info:**
            - ``<directory_path>`` is a quoted string that specifies the directory to create.
        """
        return self._mkdir

    @property
    def mount(self) -> FilesystemMount:
        """Return the ``FILESystem:MOUNT`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:MOUNT?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:MOUNT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.drive``: The ``FILESystem:MOUNT:DRIVE`` command.
            - ``.tekdrive``: The ``FILESystem:MOUNT:TEKDrive`` command.
        """
        return self._mount

    @property
    def readfile(self) -> FilesystemReadfile:
        """Return the ``FILESystem:READFile`` command.

        **Description:**
            - This command writes the contents of the specified file to the current interface. If
              the specified file does not exist or is not readable, an appropriate error event is
              posted.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:READFile value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:READFile <QString>

        **Info:**
            - ``<QString>`` is a quoted string that defines the file name and path. If the file path
              is within the current working directory, you need only specify the file name.
        """
        return self._readfile

    @property
    def rename(self) -> FilesystemRename:
        """Return the ``FILESystem:REName`` command.

        **Description:**
            - This command (no query form) assigns a new name to an existing file or folder.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:REName value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:REName <old_file_path>,<new_file_path>

        **Info:**
            - ``<old_file_path>`` is a quoted string that defines the file or folder name and path.
              If the path is within the current working directory, you need only specify the file or
              folder name.
            - ``<new_file_path>`` is a quoted string that defines the file or folder name and path.
              If the path is within the current working directory, you need only specify the file or
              folder name.
        """
        return self._rename

    @property
    def rmdir(self) -> FilesystemRmdir:
        """Return the ``FILESystem:RMDir`` command.

        **Description:**
            - This command (no query form) deletes a named directory. The directory must be empty.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:RMDir value`` command.

        **SCPI Syntax:**

        ::

            - FILESystem:RMDir <directory_path>

        **Info:**
            - ``<directory_path>`` is a quoted string that defines the folder name and path. If the
              folder path is within the current working directory, you need only specify the folder
              name.
        """
        return self._rmdir

    @property
    def tekdrive(self) -> FilesystemTekdrive:
        """Return the ``FILESystem:TEKDrive`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:TEKDrive?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:TEKDrive?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.code``: The ``FILESystem:TEKDrive:CODE`` command.
        """
        return self._tekdrive

    @property
    def unmount(self) -> FilesystemUnmount:
        """Return the ``FILESystem:UNMOUNT`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``FILESystem:UNMOUNT?`` query.
            - Using the ``.verify(value)`` method will send the ``FILESystem:UNMOUNT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.drive``: The ``FILESystem:UNMOUNT:DRIve`` command.
            - ``.tekdrive``: The ``FILESystem:UNMOUNT:TEKDrive`` command.
        """
        return self._unmount

    @property
    def writefile(self) -> FilesystemWritefile:
        """Return the ``FILESystem:WRITEFile`` command.

        **Description:**
            - This command (no query form) writes the specified block data to the specified file on
              the instruments file system. If the destination file cannot be written, an error event
              is posted.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FILESystem:WRITEFile value``
              command.

        **SCPI Syntax:**

        ::

            - FILESystem:WRITEFile <file_path>,<data>

        **Info:**
            - ``<file path>`` is a quoted string that defines the file name and path. If the file
              path is within the current working directory, you need only specify the file name.
            - ``<data>`` is the specified block data to be written.
        """
        return self._writefile



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The io commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2604B, SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B,
SMU2651A, SMU2657A, SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - io.close()
    - io.flush()
    - io.input()
    - io.open()
    - io.output()
    - io.read()
    - io.type()
    - io.write()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Io(BaseTSPCmd):
    """The ``io`` command tree.

    Properties/methods:
        - ``.close()``: The ``io.close()`` function.
        - ``.flush()``: The ``io.flush()`` function.
        - ``.input()``: The ``io.input()`` function.
        - ``.open()``: The ``io.open()`` function.
        - ``.output()``: The ``io.output()`` function.
        - ``.read()``: The ``io.read()`` function.
        - ``.type()``: The ``io.type()`` function.
        - ``.write()``: The ``io.write()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "io") -> None:
        super().__init__(device, cmd_syntax)

    def close(self, file: Optional[str] = None) -> None:
        """Run the ``io.close()`` function.

        **Description:**
            - This function closes a file.

        **TSP Syntax:**

        ::

            - io.close()

        Args:
            file (optional): The descriptor of the file to close.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (file,) if x is not None)
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.close({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.close()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def flush(self) -> None:
        """Run the ``io.flush()`` function.

        **Description:**
            - This function saves buffered data to a file.

        **TSP Syntax:**

        ::

            - io.flush()

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.flush()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.flush()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def input(self, newfile: Optional[str] = None) -> str:
        """Run the ``io.input()`` function.

        **Description:**
            - This function assigns a previously opened file, or opens a new file, as the default
              input file.

        **TSP Syntax:**

        ::

            - io.input()

        Args:
            newfile (optional): A string representing the path of a file to open as the default
                input file, or the file descriptor of an open file to use as the default input file.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x) for x in (f'"{newfile}"' if newfile is not None else None,) if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.input({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.input()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def open(self, path: str, mode: Optional[str] = None) -> str:
        """Run the ``io.open()`` function.

        **Description:**
            - This function opens a file for later reference.

        **TSP Syntax:**

        ::

            - io.open()

        Args:
            path: The path of the file to open.
            mode (optional): A string representing the intended access mode ('r' = read, 'w' =
                write, and 'a' = append).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{path}"',
                    f'"{mode}"' if mode is not None else None,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.open({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.open()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def output(self, newfile: Optional[str] = None) -> str:
        """Run the ``io.output()`` function.

        **Description:**
            - This function assigns a previously opened file or opens a new file as the default
              output file.

        **TSP Syntax:**

        ::

            - io.output()

        Args:
            newfile (optional): A file descriptor to assign (or the path of a file to open) as the
                default output file.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x) for x in (f'"{newfile}"' if newfile is not None else None,) if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.output({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.output()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def read(self, format_: Optional[str] = None) -> str:
        """Run the ``io.read()`` function.

        **Description:**
            - This function reads data from the default input file.

        **TSP Syntax:**

        ::

            - io.read()

        Args:
            format_ (optional): A string or number indicating the type of data to be read.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x) for x in (f'"{format_}"' if format_ is not None else None,) if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.read({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.read()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def type(self, obj: str) -> str:
        """Run the ``io.type()`` function.

        **Description:**
            - This function checks whether or not a given object is a file handle.

        **TSP Syntax:**

        ::

            - io.type()

        Args:
            obj: Object to check.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.type({obj}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.type()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def write(self, data: Optional[str] = None) -> None:
        """Run the ``io.write()`` function.

        **Description:**
            - This function writes data to the default output file.

        **TSP Syntax:**

        ::

            - io.write()

        Args:
            data (optional): The data to be written.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (data,) if x is not None)
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.write({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.write()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The acquire commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - ACQuire:FASTAcq
    - ACQuire:FASTAcq:PALEtte {NORMal|TEMPErature|SPECTral|INVERTed}
    - ACQuire:FASTAcq:PALEtte?
    - ACQuire:FASTAcq:STATE {0|1|OFF|ON}
    - ACQuire:FASTAcq:STATE?
    - ACQuire:MAGnivu {<NR1>|OFF|ON}
    - ACQuire:MAGnivu?
    - ACQuire:MAXSamplerate?
    - ACQuire:MODe {SAMple|PEAKdetect|HIRes|AVErage|ENVelope}
    - ACQuire:MODe?
    - ACQuire:NUMACq?
    - ACQuire:NUMAVg <NR1>
    - ACQuire:NUMAVg?
    - ACQuire:NUMEnv {<NR1>|INFInite}
    - ACQuire:NUMEnv?
    - ACQuire:STATE {OFF|ON|RUN|STOP|<NR1>}
    - ACQuire:STATE?
    - ACQuire:STOPAfter {RUNSTop|SEQuence}
    - ACQuire:STOPAfter?
    - ACQuire?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AcquireStopafter(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:STOPAfter`` command.

    **Description:**
        - This command sets or queries whether the instrument continually acquires acquisitions or
          acquires a single sequence. Pressing SINGLE on the front panel button is equivalent to
          sending these commands: ``ACQUIRE:STOPAFTER SEQUENCE`` and ``ACQUIRE:STATE 1``.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:STOPAfter?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:STOPAfter?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:STOPAfter value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:STOPAfter {RUNSTop|SEQuence}
        - ACQuire:STOPAfter?

    **Info:**
        - ``RUNSTop`` specifies that the instrument will continually acquire data, if
          ``ACQuire:STATE`` is turned on.
        - ``SEQuence`` specifies that the next acquisition will be a single-sequence acquisition.
    """


class AcquireState(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:STATE`` command.

    **Description:**
        - Starts or stops acquisitions. When state is set to ON or RUN, a new acquisition will be
          started. If the last acquisition was a single acquisition sequence, a new single sequence
          acquisition will be started. If the last acquisition was continuous, a new continuous
          acquisition will be started. If RUN is issued in the process of completing a single
          sequence acquisition (for example, averaging or enveloping), the acquisition sequence is
          restarted, and any accumulated data is discarded. Also, the oscilloscope resets the number
          of acquisitions. If the RUN argument is issued while in continuous mode, acquisition
          continues.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:STATE value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:STATE {OFF|ON|RUN|STOP|<NR1>}
        - ACQuire:STATE?

    **Info:**
        - ``OFF`` stops acquisitions.
        - ``STOP`` stops acquisitions.
        - ``ON`` starts acquisitions.
        - ``RUN`` starts acquisitions.
        - ``<NR1>`` = 0 stops acquisitions; any other value starts acquisitions.
    """


class AcquireNumenv(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:NUMEnv`` command.

    **Description:**
        - This command controls the number of envelopes (when acquisition mode has been set to
          ENVelope using ``ACQUIRE:MODE``). The number of envelopes can be set from 1 to 2000 in
          increments of 1, or to INFInite. Setting the value to a number greater than 2000 sets the
          number of envelopes to INFInite.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:NUMEnv?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:NUMEnv?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:NUMEnv value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:NUMEnv {<NR1>|INFInite}
        - ACQuire:NUMEnv?

    **Info:**
        - ``<NR1>`` is an integer that specifies the number of envelopes to use when the acquisition
          mode has been set to ENVelope.
        - ``INFInite`` specifies to use an infinite number of envelopes.
    """


class AcquireNumavg(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:NUMAVg`` command.

    **Description:**
        - This command sets or queries the number of waveform acquisitions that make up an averaged
          waveform. Ranges from 2 to 10240.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:NUMAVg?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:NUMAVg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:NUMAVg value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:NUMAVg <NR1>
        - ACQuire:NUMAVg?

    **Info:**
        - ``<NR1>`` is the number of waveform acquisitions to average.
    """


class AcquireNumacq(SCPICmdRead):
    """The ``ACQuire:NUMACq`` command.

    **Description:**
        - This query-only command returns the number of waveform acquisitions that have occurred
          since the last time acquisitions were stopped.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:NUMACq?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:NUMACq?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - ACQuire:NUMACq?
    """


class AcquireMode(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:MODe`` command.

    **Description:**
        - This command sets or queries the selected acquisition mode of the instrument.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:MODe value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:MODe {SAMple|PEAKdetect|HIRes|AVErage|ENVelope}
        - ACQuire:MODe?

    **Info:**
        - ``SAMple`` specifies that the displayed data point value is the first sampled value that
          is taken during the acquisition interval. The CURVE query, depending on sample rate, will
          result in either 8 bit or 16 bit data. In case of 8 bit data, the precision is also 8 bit.
          However, in case of 16 bit data, 12 bit precision data is zero padded in lower 4 bits.
          SAMple is the default mode.
        - ``PEAKdetect`` specifies the display of high-low range of the samples taken from a single
          waveform acquisition. The high-low range is displayed as a vertical column that extends
          from the highest to the lowest value sampled during the acquisition interval. PEAKdetect
          mode can reveal the presence of aliasing or narrow spikes.
        - ``HIRes`` specifies Hi Res mode where the displayed data point value is the average of all
          the samples taken during the acquisition interval. This is a form of averaging, where the
          average comes from a single waveform acquisition. The number of samples taken during the
          acquisition interval determines the number of data values that compose the average.
        - ``AVErage`` specifies averaging mode, in which the resulting waveform shows an average of
          SAMple data points from several separate waveform acquisitions. The instrument processes
          the number of waveforms you specify into the acquired waveform, creating a running
          exponential average of the input signal. The number of waveform acquisitions that go into
          making up the average waveform is set or queried using the ``ACQUIRE:NUMAVG`` command.
        - ``ENVelope`` specifies envelope mode, where the resulting waveform displays the range of
          PEAKdetect from continued waveform acquisitions.
    """


class AcquireMaxsamplerate(SCPICmdRead):
    """The ``ACQuire:MAXSamplerate`` command.

    **Description:**
        - This query returns the maximum real-time sample rate.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:MAXSamplerate?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:MAXSamplerate?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - ACQuire:MAXSamplerate?
    """


class AcquireMagnivu(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:MAGnivu`` command.

    **Description:**
        - Turns on the MagniVu feature, which provides up to 32 times signal detail for fast viewing
          of short events. This feature is not recommended for slow data formats such as RS-232.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:MAGnivu?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:MAGnivu?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:MAGnivu value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:MAGnivu {<NR1>|OFF|ON}
        - ACQuire:MAGnivu?

    **Info:**
        - ``<NR1> = 0`` disables the MagniVu feature; any other value turns this feature on.
        - ``ON`` enables the MagniVu feature.
        - ``OFF`` disables the MagniVu feature.
    """


class AcquireFastacqState(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:FASTAcq:STATE`` command.

    **Description:**
        - Turns fast acquisition mode on or off, or queries the state of the mode.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:FASTAcq:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAcq:STATE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:FASTAcq:STATE value`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:FASTAcq:STATE {0|1|OFF|ON}
        - ACQuire:FASTAcq:STATE?

    **Info:**
        - ``1`` or ON turns on fast acquisition mode.
        - ``0`` or OFF turns it off.
    """


class AcquireFastacqPalette(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:FASTAcq:PALEtte`` command.

    **Description:**
        - Sets (or queries) which palette to use for fast acquisition mode.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire:FASTAcq:PALEtte?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAcq:PALEtte?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:FASTAcq:PALEtte value``
          command.

    **SCPI Syntax:**

    ::

        - ACQuire:FASTAcq:PALEtte {NORMal|TEMPErature|SPECTral|INVERTed}
        - ACQuire:FASTAcq:PALEtte?

    **Info:**
        - ``NORMal`` - Normal displays hues and lightness levels for best overall viewing. The color
          of each channel waveform matches the color of the corresponding front-panel vertical knob.
        - ``TEMPErature`` - Temperature Grading displays areas of the waveform with the highest
          sample density in red shades. The areas of lowest sample density appear in blue shades.
        - ``SPECTra`` - Spectral Grading displays areas of the waveform with the highest sample
          density in blue shades. The areas of lowest sample density appear in red shades.
        - ``INVERTed`` - Inverts the normal display hues and lightness levels based on sample
          intensity. The areas of lowest sample density appear the brightest, while the areas with
          the highest sample density appear the darkest.
    """


class AcquireFastacq(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``ACQuire:FASTAcq`` command.

    **Description:**
        - Sets or queries the FastAcq feature. This feature provides a high-speed waveform capture
          rate to help capture signal anomalies.

    **Usage:**
        - Using the ``.write()`` method will send the ``ACQuire:FASTAcq`` command.

    **SCPI Syntax:**

    ::

        - ACQuire:FASTAcq

    Properties:
        - ``.palette``: The ``ACQuire:FASTAcq:PALEtte`` command.
        - ``.state``: The ``ACQuire:FASTAcq:STATE`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._palette = AcquireFastacqPalette(device, f"{self._cmd_syntax}:PALEtte")
        self._state = AcquireFastacqState(device, f"{self._cmd_syntax}:STATE")

    @property
    def palette(self) -> AcquireFastacqPalette:
        """Return the ``ACQuire:FASTAcq:PALEtte`` command.

        **Description:**
            - Sets (or queries) which palette to use for fast acquisition mode.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:FASTAcq:PALEtte?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAcq:PALEtte?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:FASTAcq:PALEtte value``
              command.

        **SCPI Syntax:**

        ::

            - ACQuire:FASTAcq:PALEtte {NORMal|TEMPErature|SPECTral|INVERTed}
            - ACQuire:FASTAcq:PALEtte?

        **Info:**
            - ``NORMal`` - Normal displays hues and lightness levels for best overall viewing. The
              color of each channel waveform matches the color of the corresponding front-panel
              vertical knob.
            - ``TEMPErature`` - Temperature Grading displays areas of the waveform with the highest
              sample density in red shades. The areas of lowest sample density appear in blue
              shades.
            - ``SPECTra`` - Spectral Grading displays areas of the waveform with the highest sample
              density in blue shades. The areas of lowest sample density appear in red shades.
            - ``INVERTed`` - Inverts the normal display hues and lightness levels based on sample
              intensity. The areas of lowest sample density appear the brightest, while the areas
              with the highest sample density appear the darkest.
        """
        return self._palette

    @property
    def state(self) -> AcquireFastacqState:
        """Return the ``ACQuire:FASTAcq:STATE`` command.

        **Description:**
            - Turns fast acquisition mode on or off, or queries the state of the mode.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:FASTAcq:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAcq:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:FASTAcq:STATE value``
              command.

        **SCPI Syntax:**

        ::

            - ACQuire:FASTAcq:STATE {0|1|OFF|ON}
            - ACQuire:FASTAcq:STATE?

        **Info:**
            - ``1`` or ON turns on fast acquisition mode.
            - ``0`` or OFF turns it off.
        """
        return self._state


#  pylint: disable=too-many-instance-attributes
class Acquire(SCPICmdRead):
    """The ``ACQuire`` command.

    **Description:**
        - Queries the current acquisition state.

    **Usage:**
        - Using the ``.query()`` method will send the ``ACQuire?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - ACQuire?

    Properties:
        - ``.fastacq``: The ``ACQuire:FASTAcq`` command.
        - ``.magnivu``: The ``ACQuire:MAGnivu`` command.
        - ``.maxsamplerate``: The ``ACQuire:MAXSamplerate`` command.
        - ``.mode``: The ``ACQuire:MODe`` command.
        - ``.numacq``: The ``ACQuire:NUMACq`` command.
        - ``.numavg``: The ``ACQuire:NUMAVg`` command.
        - ``.numenv``: The ``ACQuire:NUMEnv`` command.
        - ``.state``: The ``ACQuire:STATE`` command.
        - ``.stopafter``: The ``ACQuire:STOPAfter`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "ACQuire") -> None:
        super().__init__(device, cmd_syntax)
        self._fastacq = AcquireFastacq(device, f"{self._cmd_syntax}:FASTAcq")
        self._magnivu = AcquireMagnivu(device, f"{self._cmd_syntax}:MAGnivu")
        self._maxsamplerate = AcquireMaxsamplerate(device, f"{self._cmd_syntax}:MAXSamplerate")
        self._mode = AcquireMode(device, f"{self._cmd_syntax}:MODe")
        self._numacq = AcquireNumacq(device, f"{self._cmd_syntax}:NUMACq")
        self._numavg = AcquireNumavg(device, f"{self._cmd_syntax}:NUMAVg")
        self._numenv = AcquireNumenv(device, f"{self._cmd_syntax}:NUMEnv")
        self._state = AcquireState(device, f"{self._cmd_syntax}:STATE")
        self._stopafter = AcquireStopafter(device, f"{self._cmd_syntax}:STOPAfter")

    @property
    def fastacq(self) -> AcquireFastacq:
        """Return the ``ACQuire:FASTAcq`` command.

        **Description:**
            - Sets or queries the FastAcq feature. This feature provides a high-speed waveform
              capture rate to help capture signal anomalies.

        **Usage:**
            - Using the ``.write()`` method will send the ``ACQuire:FASTAcq`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:FASTAcq

        Sub-properties:
            - ``.palette``: The ``ACQuire:FASTAcq:PALEtte`` command.
            - ``.state``: The ``ACQuire:FASTAcq:STATE`` command.
        """
        return self._fastacq

    @property
    def magnivu(self) -> AcquireMagnivu:
        """Return the ``ACQuire:MAGnivu`` command.

        **Description:**
            - Turns on the MagniVu feature, which provides up to 32 times signal detail for fast
              viewing of short events. This feature is not recommended for slow data formats such as
              RS-232.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:MAGnivu?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:MAGnivu?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:MAGnivu value`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:MAGnivu {<NR1>|OFF|ON}
            - ACQuire:MAGnivu?

        **Info:**
            - ``<NR1> = 0`` disables the MagniVu feature; any other value turns this feature on.
            - ``ON`` enables the MagniVu feature.
            - ``OFF`` disables the MagniVu feature.
        """
        return self._magnivu

    @property
    def maxsamplerate(self) -> AcquireMaxsamplerate:
        """Return the ``ACQuire:MAXSamplerate`` command.

        **Description:**
            - This query returns the maximum real-time sample rate.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:MAXSamplerate?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:MAXSamplerate?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - ACQuire:MAXSamplerate?
        """
        return self._maxsamplerate

    @property
    def mode(self) -> AcquireMode:
        """Return the ``ACQuire:MODe`` command.

        **Description:**
            - This command sets or queries the selected acquisition mode of the instrument.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:MODe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:MODe value`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:MODe {SAMple|PEAKdetect|HIRes|AVErage|ENVelope}
            - ACQuire:MODe?

        **Info:**
            - ``SAMple`` specifies that the displayed data point value is the first sampled value
              that is taken during the acquisition interval. The CURVE query, depending on sample
              rate, will result in either 8 bit or 16 bit data. In case of 8 bit data, the precision
              is also 8 bit. However, in case of 16 bit data, 12 bit precision data is zero padded
              in lower 4 bits. SAMple is the default mode.
            - ``PEAKdetect`` specifies the display of high-low range of the samples taken from a
              single waveform acquisition. The high-low range is displayed as a vertical column that
              extends from the highest to the lowest value sampled during the acquisition interval.
              PEAKdetect mode can reveal the presence of aliasing or narrow spikes.
            - ``HIRes`` specifies Hi Res mode where the displayed data point value is the average of
              all the samples taken during the acquisition interval. This is a form of averaging,
              where the average comes from a single waveform acquisition. The number of samples
              taken during the acquisition interval determines the number of data values that
              compose the average.
            - ``AVErage`` specifies averaging mode, in which the resulting waveform shows an average
              of SAMple data points from several separate waveform acquisitions. The instrument
              processes the number of waveforms you specify into the acquired waveform, creating a
              running exponential average of the input signal. The number of waveform acquisitions
              that go into making up the average waveform is set or queried using the
              ``ACQUIRE:NUMAVG`` command.
            - ``ENVelope`` specifies envelope mode, where the resulting waveform displays the range
              of PEAKdetect from continued waveform acquisitions.
        """
        return self._mode

    @property
    def numacq(self) -> AcquireNumacq:
        """Return the ``ACQuire:NUMACq`` command.

        **Description:**
            - This query-only command returns the number of waveform acquisitions that have occurred
              since the last time acquisitions were stopped.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:NUMACq?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:NUMACq?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - ACQuire:NUMACq?
        """
        return self._numacq

    @property
    def numavg(self) -> AcquireNumavg:
        """Return the ``ACQuire:NUMAVg`` command.

        **Description:**
            - This command sets or queries the number of waveform acquisitions that make up an
              averaged waveform. Ranges from 2 to 10240.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:NUMAVg?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:NUMAVg?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:NUMAVg value`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:NUMAVg <NR1>
            - ACQuire:NUMAVg?

        **Info:**
            - ``<NR1>`` is the number of waveform acquisitions to average.
        """
        return self._numavg

    @property
    def numenv(self) -> AcquireNumenv:
        """Return the ``ACQuire:NUMEnv`` command.

        **Description:**
            - This command controls the number of envelopes (when acquisition mode has been set to
              ENVelope using ``ACQUIRE:MODE``). The number of envelopes can be set from 1 to 2000 in
              increments of 1, or to INFInite. Setting the value to a number greater than 2000 sets
              the number of envelopes to INFInite.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:NUMEnv?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:NUMEnv?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:NUMEnv value`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:NUMEnv {<NR1>|INFInite}
            - ACQuire:NUMEnv?

        **Info:**
            - ``<NR1>`` is an integer that specifies the number of envelopes to use when the
              acquisition mode has been set to ENVelope.
            - ``INFInite`` specifies to use an infinite number of envelopes.
        """
        return self._numenv

    @property
    def state(self) -> AcquireState:
        """Return the ``ACQuire:STATE`` command.

        **Description:**
            - Starts or stops acquisitions. When state is set to ON or RUN, a new acquisition will
              be started. If the last acquisition was a single acquisition sequence, a new single
              sequence acquisition will be started. If the last acquisition was continuous, a new
              continuous acquisition will be started. If RUN is issued in the process of completing
              a single sequence acquisition (for example, averaging or enveloping), the acquisition
              sequence is restarted, and any accumulated data is discarded. Also, the oscilloscope
              resets the number of acquisitions. If the RUN argument is issued while in continuous
              mode, acquisition continues.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:STATE?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:STATE value`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:STATE {OFF|ON|RUN|STOP|<NR1>}
            - ACQuire:STATE?

        **Info:**
            - ``OFF`` stops acquisitions.
            - ``STOP`` stops acquisitions.
            - ``ON`` starts acquisitions.
            - ``RUN`` starts acquisitions.
            - ``<NR1>`` = 0 stops acquisitions; any other value starts acquisitions.
        """
        return self._state

    @property
    def stopafter(self) -> AcquireStopafter:
        """Return the ``ACQuire:STOPAfter`` command.

        **Description:**
            - This command sets or queries whether the instrument continually acquires acquisitions
              or acquires a single sequence. Pressing SINGLE on the front panel button is equivalent
              to sending these commands: ``ACQUIRE:STOPAFTER SEQUENCE`` and ``ACQUIRE:STATE 1``.

        **Usage:**
            - Using the ``.query()`` method will send the ``ACQuire:STOPAfter?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:STOPAfter?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:STOPAfter value`` command.

        **SCPI Syntax:**

        ::

            - ACQuire:STOPAfter {RUNSTop|SEQuence}
            - ACQuire:STOPAfter?

        **Info:**
            - ``RUNSTop`` specifies that the instrument will continually acquire data, if
              ``ACQuire:STATE`` is turned on.
            - ``SEQuence`` specifies that the next acquisition will be a single-sequence
              acquisition.
        """
        return self._stopafter



"""Base Source Measure Unit (SMU) device driver module.

SMUs include TSP devices and PI devices.
"""
from abc import ABC

from tm_devices.drivers.pi.tsp_device import TSPDevice
from tm_devices.helpers import DeviceTypes


class SourceMeasureUnit(TSPDevice, ABC):
    """Base Source Measure Unit (SMU) device driver."""

    _DEVICE_TYPE = DeviceTypes.SMU.value

    ################################################################################################
    # Public Methods
    ################################################################################################



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The bit commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2604B, SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B,
SMU2651A, SMU2657A, SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - bit.bitand()
    - bit.bitor()
    - bit.bitxor()
    - bit.clear()
    - bit.get()
    - bit.getfield()
    - bit.set()
    - bit.setfield()
    - bit.test()
    - bit.toggle()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Bit(BaseTSPCmd):
    """The ``bit`` command tree.

    Properties/methods:
        - ``.bitand()``: The ``bit.bitand()`` function.
        - ``.bitor()``: The ``bit.bitor()`` function.
        - ``.bitxor()``: The ``bit.bitxor()`` function.
        - ``.clear()``: The ``bit.clear()`` function.
        - ``.get()``: The ``bit.get()`` function.
        - ``.getfield()``: The ``bit.getfield()`` function.
        - ``.set()``: The ``bit.set()`` function.
        - ``.setfield()``: The ``bit.setfield()`` function.
        - ``.test()``: The ``bit.test()`` function.
        - ``.toggle()``: The ``bit.toggle()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "bit") -> None:
        super().__init__(device, cmd_syntax)

    def bitand(self, value1: str, value2: str) -> str:
        """Run the ``bit.bitand()`` function.

        **Description:**
            - This function performs a bitwise logical AND operation on two numbers.

        **TSP Syntax:**

        ::

            - bit.bitand()

        Args:
            value1: Operand for the logical AND operation.
            value2: Operand for the logical AND operation.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.bitand({value1}, {value2}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.bitand()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def bitor(self, value1: str, value2: str) -> str:
        """Run the ``bit.bitor()`` function.

        **Description:**
            - This function performs a bitwise logical OR operation on two numbers.

        **TSP Syntax:**

        ::

            - bit.bitor()

        Args:
            value1: Operand for the logical OR operation.
            value2: Operand for the logical OR operation.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.bitor({value1}, {value2}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.bitor()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def bitxor(self, value1: str, value2: str) -> str:
        """Run the ``bit.bitxor()`` function.

        **Description:**
            - This function performs a bitwise logical XOR (exclusive OR) operation on two numbers.

        **TSP Syntax:**

        ::

            - bit.bitxor()

        Args:
            value1: Operand for the logical XOR operation.
            value2: Operand for the logical XOR operation.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.bitxor({value1}, {value2}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.bitxor()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clear(self, value: str, index: int) -> str:
        """Run the ``bit.clear()`` function.

        **Description:**
            - This function clears a bit at a specified index position.

        **TSP Syntax:**

        ::

            - bit.clear()

        Args:
            value: Specified number.
            index: One-based bit position within value to clear (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.clear({value}, {index}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def get(self, value: str, index: int) -> str:
        """Run the ``bit.get()`` function.

        **Description:**
            - This function retrieves the weighted value of a bit at a specified index position.

        **TSP Syntax:**

        ::

            - bit.get()

        Args:
            value: Specified number.
            index: One-based bit position within value to get (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.get({value}, {index}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.get()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def getfield(self, value: str, index: int, width: int) -> str:
        """Run the ``bit.getfield()`` function.

        **Description:**
            - This function returns a field of bits from the value starting at the specified index
              position.

        **TSP Syntax:**

        ::

            - bit.getfield()

        Args:
            value: Specified number.
            index: One-based bit position within value to get (1 to 32).
            width: The number of bits to include in the field (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.getfield({value}, {index}, {width}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.getfield()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def set_(self, value: str, index: int) -> str:
        """Run the ``bit.set()`` function.

        **Description:**
            - This function sets a bit at the specified index position.

        **TSP Syntax:**

        ::

            - bit.set()

        Args:
            value: Specified number.
            index: One-based bit position within value to set (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.set({value}, {index}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.set()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def setfield(self, value: str, index: int, width: int, field_value: str) -> str:
        """Run the ``bit.setfield()`` function.

        **Description:**
            - This function overwrites a bit field at a specified index position.

        **TSP Syntax:**

        ::

            - bit.setfield()

        Args:
            value: Specified number.
            index: One-based bit position in value to set (1 to 32).
            width: The number of bits to include in the field (1 to 32).
            field_value: Value to write to the field.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.setfield({value}, {index}, {width}, {field_value}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.setfield()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def test(self, value: str, index: int) -> str:
        """Run the ``bit.test()`` function.

        **Description:**
            - This function returns the Boolean value (true or false) of a bit at the specified
              index position.

        **TSP Syntax:**

        ::

            - bit.test()

        Args:
            value: Specified number.
            index: One-based bit position within value to test (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.test({value}, {index}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.test()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def toggle(self, value: str, index: int) -> str:
        """Run the ``bit.toggle()`` function.

        **Description:**
            - This function toggles the value of a bit at a specified index position.

        **TSP Syntax:**

        ::

            - bit.toggle()

        Args:
            value: Specified number.
            index: One-based bit position within value to toggle (1 to 32).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.toggle({value}, {index}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.toggle()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error
